type StringAndOthers<T extends unknown[]> = [string, ...T];
interface SchemaField {
    verbose_name: string;
    class: string;
    choices: null | Record<string, string>;
}
type SchemaModelRelType = {
    model: string;
    verbose_name: string | null;
};
type SchemaModel = {
    __rel: Record<string, SchemaModelRelType>;
    __verbose_name: string;
    [key: string]: SchemaField | string | Record<string, SchemaModelRelType>;
};
interface SchemaModels {
    [key: string]: SchemaModel;
}
interface SchemaLookups {
    [key: string]: string[];
}
interface Schema {
    models: SchemaModels;
    lookups: SchemaLookups;
}
type LispyFunction = StringAndOthers<unknown[]>;
type LispyConditionExpr = readonly ['=', string, string | number | boolean | LispyFunction];
type LispyNotExpr = ['not', LispyExpression];
type LispyAndOrExpr = ['and' | 'or', ...LispyExpression[]];
type LispyExpression = LispyNotExpr | LispyAndOrExpr | LispyConditionExpr;
type LispyOperator = 'not' | 'and' | 'or';
interface FilterEditorOptions {
    container: HTMLElement | string;
    schema: any;
    mainModel: string;
    initialExpression?: LispyExpression;
    getValueInputFn?: (lookup: string, field: SchemaField, initialValue: string | null) => HTMLElement;
}
declare function isLispyOperator(value: unknown): value is LispyOperator;
declare function isLispyConditionExpr(expr: unknown): expr is LispyConditionExpr;
declare function isLispyNotExpr(expr: unknown): expr is LispyNotExpr;
declare function isLispyAndOrExpr(expr: unknown): expr is LispyAndOrExpr;
declare function isLispyExpression(expr: unknown): expr is LispyExpression;
declare function isSchemaField(obj: any): obj is SchemaField;

declare function getValueInput(lookup: string, field: SchemaField, initialValue: string | null): HTMLSelectElement | HTMLInputElement;
declare class FilterEditor {
    private container;
    private expression;
    private schema;
    private mainModel;
    private fieldsList;
    private getValueInputFn;
    constructor(options: FilterEditorOptions);
    private init;
    private setupSortableForGroup;
    /**
     * setup a new CascaderSelect
     */
    private setupNewCascaderSelect;
    private setupLookupSelector;
    private setupValueInput;
    private render;
    private renderExpression;
    private renderOperator;
    private renderCondition;
    private renderCommonActions;
    private setupEventListener;
    private actionDelete;
    private actionAddNew;
    private addNewGroup;
    private addNewCondition;
    private actionToggleNot;
    toJson(): LispyExpression;
    private parseGroup;
    private parseCondition;
}

export { FilterEditor, type FilterEditorOptions, type LispyConditionExpr, type LispyExpression, type LispyFunction, type LispyOperator, type Schema, type SchemaField, type SchemaLookups, type SchemaModel, type SchemaModelRelType, type SchemaModels, getValueInput, isLispyAndOrExpr, isLispyConditionExpr, isLispyExpression, isLispyNotExpr, isLispyOperator, isSchemaField };
